<?php

namespace App\Controller\OgBoot\OgLive\Webhook;

use App\Controller\OgBoot\AbstractOgBootController;
use App\Entity\OgLive;
use App\Model\OgLiveStatus;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

#[AsController]
class InstallOgLiveResponseAction extends AbstractController
{
    public CONST string OG_LIVE_INSTALL_SUCCESS = 'success';
    public CONST string OG_LIVE_INSTALL_FAILED = 'failure';
    const string OG_BOOT_DIRECTORY = '/opt/opengnsys/ogboot/tftpboot//';

    public function __construct(
        protected readonly EntityManagerInterface $entityManager
    )
    {
    }

    #[Route('/og-lives/install/webhook', name: 'install_webhook', methods: ['POST'])]
    public function installWebhook(Request $request): JsonResponse
    {
        $data = json_decode($request->getContent(), true);

        if (!is_array($data)) {
            return new JsonResponse(['error' => 'Invalid JSON data'], Response::HTTP_BAD_REQUEST);
        }

        $data = $data['webhookData'];

        if ($data === null || !isset($data['message'], $data['ogCoreId'], $data['status'])) {
            return new JsonResponse(['error' => 'Invalid or incomplete JSON data'], Response::HTTP_BAD_REQUEST);
        }

        $message = $data['message'];
        $ogCoreId = $data['ogCoreId'];
        $status = $data['status'];

        $ogLive = $this->entityManager->getRepository(OgLive::class)->findOneBy(['uuid' => $ogCoreId]);

        if (!$ogLive) {
            return new JsonResponse(['error' => 'OgLive not found'], Response::HTTP_NOT_FOUND);
        }

        if ($ogLive->getStatus() === OgLiveStatus::ACTIVE) {
            return new JsonResponse(['error' => 'OgLive is already active'], Response::HTTP_BAD_REQUEST);
        }

        $this->updateOgLive($ogLive, $message, $status);

        return new JsonResponse(data: sprintf('OgLive %s updated successfully', $ogLive->getChecksum()), status: Response::HTTP_OK);
    }

    private function updateOgLive (OgLive $ogLive, array $details, string $status): void
    {
        if ($status === self::OG_LIVE_INSTALL_SUCCESS) {
            $ogLive->setFilename(str_replace(self::OG_BOOT_DIRECTORY, '', $ogLive['directory']));
            $ogLive->setInstalled(true);
            $ogLive->setSynchronized(true);
            $ogLive->setChecksum($details['id']);
            $ogLive->setDistribution($details['distribution']);
            $ogLive->setKernel($details['kernel']);
            $ogLive->setArchitecture($details['architecture']);
            $ogLive->setRevision($details['revision']);
            $ogLive->setDirectory($details['directory']);
        }

        $ogLive->setStatus($status === self::OG_LIVE_INSTALL_SUCCESS ? OgLiveStatus::ACTIVE : OgLiveStatus::FAILED);
        $ogLive->setInstalled($status === self::OG_LIVE_INSTALL_SUCCESS);

        $oldDefaultOgLive = $this->entityManager->getRepository(OgLive::class)->findBy(['isDefault' => true]);

        foreach ($oldDefaultOgLive as $oldOgLive) {
            $oldOgLive->setIsDefault(false);
            $this->entityManager->persist($oldOgLive);
        }

        $ogLive->setIsDefault(true);
        $this->entityManager->persist($ogLive);

        $this->entityManager->flush();
    }
}